/*	Public domain	*/

/* Sync with csoftadmd. */
#define ADDRESS_MAX		128
#define MAILBOX_PATH_MAX	128
#define RULE_COND_MAX		128
#define RULE_INSN_MAX		256
#define RULESET_MAX		1024
#define RULE_MAX_RECURSION_LVL	32
#define SPAM_TEST_SIZE_LIMIT	131072

typedef struct sf_msg_status {
	float score;			/* SpamAssassin Score */
	float req_score;		/* Spam threshold */
	int spam_status;		/* Spam status (1=spam) */
#ifdef HAVE_SA
	SV *sv;				/* PerMsgStatus object */
#endif
} SF_MsgStatus;

typedef struct sf_message {
	char	 mail_from[ADDRESS_MAX];	/* Sender */
	char	 rcpt_to[ADDRESS_MAX];		/* Recipient */
	char	 user_part[ADDRESS_MAX];	/* Recipient local@ part */
	char	 domain_part[ADDRESS_MAX];	/* Recipient @domain.ext part */
	char	*text;				/* Message text */
	size_t	 text_len;			/* Message length (w/o NUL) */
#ifdef HAVE_SA
	SV	*svParsed;			/* SA parsed message */
	SV	*svRewrite;			/* SA rewritten message (tmp) */
#endif
	SF_MsgStatus status;			/* Test results */
	ML_MailListReq *ml;			/* Mailing list request info */
} SF_Message;

typedef struct sf_rule {
	int flags;
#define RULE_NEGATE	0x01		/* Negate the condition */
#define RULE_SMTP	0x02		/* SMTP policy server context */
	uid_t uid;			/* User */
	gid_t gid;			/* Group */
	char *cond;			/* Condition */
	char *insn;			/* Instruction */
	TAILQ_ENTRY(sf_rule) rules;
} SF_Rule;

typedef struct sf_ruleset {
	char key[ADDRESS_MAX];
	TAILQ_HEAD(,sf_rule) rules;
} SF_Ruleset;

void		 SF_MessageInit(SF_Message *);
SF_Message	*SF_MessageNew(void);
void		 SF_MessageFree(SF_Message *);
void		 SF_MessageProcess(SF_Message *);
int		 SF_MessageClassify(SF_Message *, SF_Ruleset *, int);
__inline__ int	 SF_ParseRecipientParts(const char *, char *, char *);
SF_Ruleset	*SF_RulesetParse(const char *, char *);
void		 SF_RulesetFree(SF_Ruleset *);
