/*
 * Copyright (c) 2006 Hypertriton, Inc. <http://www.hypertriton.com/>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/stat.h>
#include <sys/types.h>

#include <string.h>
#include <errno.h>
#include <syslog.h>
#include <unistd.h>
#include <stdlib.h>
#include <dirent.h>

#include "mailprocd.h"
#include "pathnames.h"

pid_t qmgrProc = 0;

int
QMGR_Init(void)
{
	struct stat sb;

	if (stat(_PATH_QUEUE, &sb) != 0 &&
	    mkdir(_PATH_QUEUE, 0700) == -1) {
		SF_SetError("mkdir %s: %s", _PATH_QUEUE, strerror(errno));
		return (-1);
	}
	return (0);
}

/* Fork the queue manager process. */
void
QMGR_ForkInstance(void)
{
	pid_t pid;

	if ((pid = fork()) != 0) {
		fprintf(sfLog, "Started qmgr process (pid %d)\n", pid);
		qmgrProc = pid;
	} else {
		fclose(stdin);
		fclose(stdout);
		fclose(stderr);
		SF_EnterServerProc("mailprocd");
		Setproctitle("qmgr");
		QMGR_Process();
		SF_ExitServerProc(0);
	}
}

/* Return the number of items in the queue. */
int
QMGR_Todo(void)
{
	DIR *dir;
	int count = 0;

	if ((dir = opendir(_PATH_QUEUE)) == NULL) {
		syslog(LOG_ERR, "opendir %s: %m", _PATH_QUEUE);
		return (0);
	}
	while (readdir(dir) != NULL) { count++; }
	closedir(dir);
	return (count>2 ? count-2 : 0);
}

/* Process all items in the queue. */
void
QMGR_Process(void)
{
	DIR *dir;
	struct dirent *dp;

	if ((dir = opendir(_PATH_QUEUE)) == NULL) {
		syslog(LOG_ERR, "opendir %s: %m", _PATH_QUEUE);
		return;
	}
	while ((dp = readdir(dir)) != NULL) {
		if (dp->d_name[0] == '.') {
			continue;
		}
		syslog(LOG_ERR, "Processing: %s", dp->d_name);
	}
	closedir(dir);
}
